@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2009 - 2017 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2009 - 2018 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -11,9 +11,11 @@ from __future__ import unicode_literals
 
 import re
 
-from PyQt5.QtCore import Qt, QRegExp, QUrl, qVersion
+from PyQt5.QtCore import Qt, QRegExp, QUrl
 from PyQt5.QtNetwork import QNetworkRequest
 
+from Globals import qVersionTuple
+
 
 # Qt version < 4.8 has an issue; it will wrongly
 # count .co.uk (and others) as second-level domains
@@ -24,7 +26,7 @@ def toSecondLevelDomain(url):
     @param url URL to extract domain from (QUrl)
     @return name of second level domain (string)
     """
-    if qVersion() >= "4.8.0":
+    if qVersionTuple() >= (4, 8, 0):
         topLevelDomain = url.topLevelDomain()
         urlHost = url.host()
         
@@ -55,11 +57,11 @@ class AdBlockRule(object):
     """
     Class implementing the AdBlock rule.
     """
-    def __init__(self, filter="", subscription=None):
+    def __init__(self, filterRule="", subscription=None):
         """
         Constructor
         
-        @param filter filter string of the rule (string)
+        @param filterRule filter string of the rule (string)
         @param subscription reference to the subscription object
             (AdBlockSubscription)
         """
@@ -90,7 +92,7 @@ class AdBlockRule(object):
         self.__elemhide = False
         self.__caseSensitivity = Qt.CaseInsensitive
         
-        self.setFilter(filter)
+        self.setFilter(filterRule)
     
     def subscription(self):
         """
@@ -108,13 +110,13 @@ class AdBlockRule(object):
         """
         return self.__filter
     
-    def setFilter(self, filter):
+    def setFilter(self, filterRule):
         """
         Public method to set the rule filter string.
         
-        @param filter rule filter string (string)
+        @param filterRule rule filter string (string)
         """
-        self.__filter = filter
+        self.__filter = filterRule
         self.__parseFilter()
     
     def __parseFilter(self):
