@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2012 - 2017 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2012 - 2018 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -14,6 +14,7 @@ import os
 from PyQt5.QtCore import pyqtSignal, QObject, QTimer, QFile, QDir, QSettings, \
     QUrl, QByteArray
 from PyQt5.QtNetwork import QNetworkAccessManager
+from PyQt5.QtWebKit import QWebFrame
 
 import Utilities
 import Preferences
@@ -61,18 +62,19 @@ class GreaseMonkeyManager(QObject):
         """
         from .GreaseMonkeyDownloader import GreaseMonkeyDownloader
         downloader = GreaseMonkeyDownloader(request, self)
-        downloader.finished.connect(self.__downloaderFinished)
+        downloader.finished.connect(
+            lambda: self.__downloaderFinished(downloader))
         self.__downloaders.append(downloader)
     
-    def __downloaderFinished(self):
+    def __downloaderFinished(self, downloader):
         """
         Private slot to handle the completion of a script download.
-        """
-        downloader = self.sender()
-        if downloader is None or downloader not in self.__downloaders:
-            return
         
-        self.__downloaders.remove(downloader)
+        @param downloader reference to the downloader object
+        @type GreaseMonkeyDownloader
+        """
+        if downloader in self.__downloaders:
+            self.__downloaders.remove(downloader)
     
     def scriptsDirectory(self):
         """
@@ -236,7 +238,7 @@ class GreaseMonkeyManager(QObject):
         Public slot to handle the start of loading a page.
         """
         frame = self.sender()
-        if not frame:
+        if frame is None or not isinstance(frame, QWebFrame):
             return
         
         urlScheme = frame.url().scheme()
