@@ -163,7 +163,7 @@ def syntaxAndPyflakesBatchCheck(argumentsList, send, fx, cancelled,
         taskQueue.put(task)
 
     # Start worker processes
-    for i in range(NumberOfProcesses):
+    for _ in range(NumberOfProcesses):
         multiprocessing.Process(target=worker, args=(taskQueue, doneQueue))\
             .start()
 
@@ -193,7 +193,7 @@ def syntaxAndPyflakesBatchCheck(argumentsList, send, fx, cancelled,
             taskQueue.put(argumentsList[i + initialTasks])
 
     # Tell child processes to stop
-    for i in range(NumberOfProcesses):
+    for _ in range(NumberOfProcesses):
         taskQueue.put('STOP')
 
 
