@@ -240,26 +240,13 @@ class ProjectBaseBrowser(Browser):
             self.vcsHelper.addVCSMenus(
                 self.mainMenu, self.multiMenu, self.backMenu,
                 self.dirMenu, self.dirMultiMenu)
-        
+    
     def _newProject(self):
         """
         Protected slot to handle the newProject signal.
         """
-        self.layoutDisplay()
-        self.sortByColumn(0, Qt.DescendingOrder)
-        self.sortByColumn(0, Qt.AscendingOrder)
-        
-        self._createPopupMenus()
-        
-        if self.backMenu is not None:
-            self.backMenu.setEnabled(True)
-        
-        if self.project.vcs is not None:
-            self.vcsHelper = self.project.vcs.vcsGetProjectBrowserHelper(
-                self, self.project, self.isTranslationsBrowser)
-            self.vcsHelper.addVCSMenus(
-                self.mainMenu, self.multiMenu, self.backMenu,
-                self.dirMenu, self.dirMultiMenu)
+        # default to perform same actions as opening a project
+        self._projectOpened()
         
     def _removeFile(self):
         """
