@@ -1539,7 +1539,7 @@ class QsciScintillaCompat(QsciScintilla):
             pos = self.SendScintilla(QsciScintilla.SCI_POSITIONFROMLINE, line)
             
             # Allow for multi-byte characters
-            for i in range(index):
+            for _ in range(index):
                 pos = self.positionAfter(pos)
             
             return pos
