@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2014 - 2017 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2014 - 2018 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -673,16 +673,17 @@ class PreviewProcessingThread(QThread):
             import markdown     # __IGNORE_EXCEPTION__
         except ImportError:
             return self.tr(
-                """<p>Markdown preview requires the <b>python-markdown</b> """
+                """<p>Markdown preview requires the <b>Markdown</b> """
                 """package.<br/>Install it with your package manager,"""
-                """ 'pip install docutils' or see """
+                """ 'pip install Markdown' or see """
                 """<a href="http://pythonhosted.org/Markdown/install.html">"""
                 """installation instructions.</a></p>""")
         
         try:
             import mdx_mathjax  # __IGNORE_EXCEPTION__ __IGNORE_WARNING__
         except ImportError:
-            #mathjax doesn't require import statement if installed as extension
+            # mathjax doesn't require import statement if installed
+            # as extension
             pass
         
         if convertNewLineToBreak:
@@ -699,9 +700,8 @@ class PreviewProcessingThread(QThread):
                 Class is placed here, because it depends on imported markdown,
                 and markdown import is lazy.
                 
-                (see http://achinghead.com/
-                python-markdown-adding-insert-delete.html this page for
-                details)
+                (see https://pythonhosted.org/Markdown/extensions/api.html
+                this page for details)
                 """
                 DEL_RE = r'(~~)(.*?)~~'
 
