@@ -299,16 +299,23 @@ def readEncodedFileWithEncoding(filename, encoding):
         return decode(text)
 
 
-def writeEncodedFile(filename, text, orig_coding):
+def writeEncodedFile(filename, text, origEncoding, forcedEncoding=""):
     """
     Function to write a file with properly encoded text.
     
-    @param filename name of the file to read (string)
-    @param text text to be written (string)
-    @param orig_coding type of the original encoding (string)
-    @return encoding used for writing the file (string)
+    @param filename name of the file to read
+    @type str
+    @param text text to be written
+    @type str
+    @param origEncoding type of the original encoding
+    @type str
+    @param forcedEncoding encoding to be used for writing, if no coding
+        line is present
+    @type str
+    @return encoding used for writing the file
+    @rtype str
     """
-    etext, encoding = encode(text, orig_coding)
+    etext, encoding = encode(text, origEncoding, forcedEncoding=forcedEncoding)
     
     f = open(filename, "wb")
     f.write(etext)
@@ -317,17 +324,23 @@ def writeEncodedFile(filename, text, orig_coding):
     return encoding
 
 
-def encode(text, orig_coding):
+def encode(text, origEncoding, forcedEncoding=""):
     """
     Function to encode text into a byte text.
     
-    @param text text to be encoded (string)
-    @param orig_coding type of the original encoding (string)
-    @return tuple of encoded text and encoding used (bytes, string)
+    @param text text to be encoded
+    @type str
+    @param origEncoding type of the original encoding
+    @type str
+    @param forcedEncoding encoding to be used for writing, if no coding line
+        is present
+    @type str
+    @return tuple of encoded text and encoding used
+    @rtype tuple of (bytes, str)
     @exception CodingError raised to indicate an invalid encoding
     """
     encoding = None
-    if orig_coding == 'utf-8-bom':
+    if origEncoding == 'utf-8-bom':
         etext, encoding = BOM_UTF8 + text.encode("utf-8"), 'utf-8-bom'
     else:
         # Try declared coding spec
@@ -339,36 +352,46 @@ def encode(text, orig_coding):
                 # Error: Declared encoding is incorrect
                 raise CodingError(coding)
         else:
-            if orig_coding and orig_coding.endswith(
-                    ('-selected', '-default', '-guessed', '-ignore')):
-                coding = orig_coding\
-                    .replace("-selected", "")\
-                    .replace("-default", "")\
-                    .replace("-guessed", "")\
-                    .replace("-ignore", "")
+            if forcedEncoding:
                 try:
-                    etext, encoding = text.encode(coding), coding
+                    etext, encoding = (
+                        text.encode(forcedEncoding), forcedEncoding)
                 except (UnicodeError, LookupError):
+                    # Error: Forced encoding is incorrect, ignore it
                     pass
             
             if encoding is None:
-                # Try configured default
-                try:
-                    codec = Preferences.getEditor("DefaultEncoding")
-                    etext, encoding = text.encode(codec), codec
-                except (UnicodeError, LookupError):
-                    pass
+                # Try the original encoding
+                if origEncoding and origEncoding.endswith(
+                        ('-selected', '-default', '-guessed', '-ignore')):
+                    coding = origEncoding\
+                        .replace("-selected", "")\
+                        .replace("-default", "")\
+                        .replace("-guessed", "")\
+                        .replace("-ignore", "")
+                    try:
+                        etext, encoding = text.encode(coding), coding
+                    except (UnicodeError, LookupError):
+                        pass
                 
                 if encoding is None:
-                    # Try saving as ASCII
+                    # Try configured default
                     try:
-                        etext, encoding = text.encode('ascii'), 'ascii'
-                    except UnicodeError:
+                        codec = Preferences.getEditor("DefaultEncoding")
+                        etext, encoding = text.encode(codec), codec
+                    except (UnicodeError, LookupError):
                         pass
                     
                     if encoding is None:
-                        # Save as UTF-8 without BOM
-                        etext, encoding = text.encode('utf-8'), 'utf-8'
+                        # Try saving as ASCII
+                        try:
+                            etext, encoding = text.encode('ascii'), 'ascii'
+                        except UnicodeError:
+                            pass
+                        
+                        if encoding is None:
+                            # Save as UTF-8 without BOM
+                            etext, encoding = text.encode('utf-8'), 'utf-8'
     
     return etext, encoding
 
@@ -568,7 +591,7 @@ def convertLineEnds(text, eol):
 
 def linesep():
     """
-    Function to return the lineseparator used by the editor.
+    Function to return the line separator used by the editor.
     
     @return line separator used by the editor (string)
     """
