@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2008 - 2017 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2008 - 2018 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 
@@ -14,12 +14,15 @@ try:
 except NameError:
     pass
 
-from PyQt5.QtCore import pyqtSlot, QUrl, QTimer, QEventLoop, QPoint, QPointF
+from PyQt5.QtCore import pyqtSlot, pyqtSignal, QUrl, QUrlQuery, QTimer, \
+    QEventLoop, QPoint, QPointF
 from PyQt5.QtGui import QDesktopServices
 from PyQt5.QtWebEngineWidgets import QWebEnginePage, QWebEngineSettings, \
     QWebEngineScript
 from PyQt5.QtWebChannel import QWebChannel
 
+from E5Gui import E5MessageBox
+
 from WebBrowser.WebBrowserWindow import WebBrowserWindow
 
 from .JavaScript.ExternalJsObject import ExternalJsObject
@@ -33,12 +36,29 @@ from Globals import qVersionTuple
 class WebBrowserPage(QWebEnginePage):
     """
     Class implementing an enhanced web page.
+    
+    @signal safeBrowsingAbort() emitted to indicate an abort due to a safe
+        browsing event
+    @signal safeBrowsingBad(threatType, threatMessages) emitted to indicate a
+        malicious web site as determined by safe browsing
+    @signal printRequested() emitted to indicate a print request of the shown
+        web page
+    @signal navigationRequestAccepted(url, navigation type, main frame) emitted
+        to signal an accepted navigation request
     """
     if qVersionTuple() >= (5, 7, 0):
-        # SafeJsWorld = QWebEngineScript.ApplicationWorld
-        SafeJsWorld = QWebEngineScript.MainWorld
+        SafeJsWorld = QWebEngineScript.ApplicationWorld
+        # SafeJsWorld = QWebEngineScript.MainWorld
     else:
         SafeJsWorld = QWebEngineScript.MainWorld
+    UnsafeJsWorld = QWebEngineScript.MainWorld
+    
+    safeBrowsingAbort = pyqtSignal()
+    safeBrowsingBad = pyqtSignal(str, str)
+    
+    printRequested = pyqtSignal()
+    navigationRequestAccepted = pyqtSignal(QUrl, QWebEnginePage.NavigationType,
+                                           bool)
     
     def __init__(self, parent=None):
         """
@@ -49,13 +69,12 @@ class WebBrowserPage(QWebEnginePage):
         super(WebBrowserPage, self).__init__(
             WebBrowserWindow.webProfile(), parent)
         
-        self.__setupWebChannel()
-        
         self.featurePermissionRequested.connect(
             self.__featurePermissionRequested)
         
         self.authenticationRequired.connect(
-            WebBrowserWindow.networkManager().authentication)
+            lambda url, auth: WebBrowserWindow.networkManager().authentication(
+                url, auth, self))
         
         self.proxyAuthenticationRequired.connect(
             WebBrowserWindow.networkManager().proxyAuthentication)
@@ -65,6 +84,23 @@ class WebBrowserPage(QWebEnginePage):
         self.urlChanged.connect(self.__urlChanged)
         
         self.__printer = None
+        self.__badSite = False
+        
+        # Workaround for changing webchannel world inside
+        # acceptNavigationRequest not working
+        self.__channelUrl = QUrl()
+        self.__channelWorldId = -1
+        self.__setupChannelTimer = QTimer(self)
+        self.__setupChannelTimer.setSingleShot(True)
+        self.__setupChannelTimer.setInterval(100)
+        self.__setupChannelTimer.timeout.connect(self.__setupChannelTimeout)
+    
+    @pyqtSlot()
+    def __setupChannelTimeout(self):
+        """
+        Private slot to initiate the setup of the web channel.
+        """
+        self.__setupWebChannelForUrl(self.__channelUrl)
     
     def acceptNavigationRequest(self, url, type_, isMainFrame):
         """
@@ -96,8 +132,67 @@ class WebBrowserPage(QWebEnginePage):
             WebBrowserWindow.greaseMonkeyManager().downloadScript(url)
             return False
         
-        return QWebEnginePage.acceptNavigationRequest(self, url, type_,
-                                                      isMainFrame)
+        if url.scheme() == "eric":
+            if url.path() == "AddSearchProvider":
+                query = QUrlQuery(url)
+                self.view().mainWindow().openSearchManager().addEngine(
+                    QUrl(query.queryItemValue("url")))
+                return False
+            elif url.path() == "PrintPage":
+                self.printRequested.emit()
+                return False
+        
+        # Safe Browsing
+        self.__badSite = False
+        from WebBrowser.SafeBrowsing.SafeBrowsingManager import \
+            SafeBrowsingManager
+        if SafeBrowsingManager.isEnabled() and \
+            url.scheme() not in \
+                SafeBrowsingManager.getIgnoreSchemes():
+            threatLists = WebBrowserWindow.safeBrowsingManager().lookupUrl(url)
+            if threatLists:
+                threatMessages = WebBrowserWindow.safeBrowsingManager()\
+                    .getThreatMessages(threatLists)
+                res = E5MessageBox.warning(
+                    WebBrowserWindow.getWindow(),
+                    self.tr("Suspicuous URL detected"),
+                    self.tr("<p>The URL <b>{0}</b> was found in the Safe"
+                            " Browsing database.</p>{1}").format(
+                        url.toString(), "".join(threatMessages)),
+                    E5MessageBox.StandardButtons(
+                        E5MessageBox.Abort |
+                        E5MessageBox.Ignore),
+                    E5MessageBox.Abort)
+                if res == E5MessageBox.Abort:
+                    self.safeBrowsingAbort.emit()
+                    return False
+                
+                self.__badSite = True
+                threatType = WebBrowserWindow.safeBrowsingManager()\
+                    .getThreatType(threatLists[0])
+                self.safeBrowsingBad.emit(threatType, "".join(threatMessages))
+        
+        result = QWebEnginePage.acceptNavigationRequest(self, url, type_,
+                                                        isMainFrame)
+        
+        if result:
+            if isMainFrame:
+                isWeb = url.scheme() in ("http", "https", "ftp", "ftps",
+                                         "file")
+                globalJsEnabled = WebBrowserWindow.webSettings().testAttribute(
+                    QWebEngineSettings.JavascriptEnabled)
+                if isWeb:
+                    enable = globalJsEnabled
+                else:
+                    enable = True
+                self.settings().setAttribute(
+                    QWebEngineSettings.JavascriptEnabled, enable)
+                
+                self.__channelUrl = url
+                self.__setupChannelTimer.start()
+            self.navigationRequestAccepted.emit(url, type_, isMainFrame)
+        
+        return result
     
     @pyqtSlot(QUrl)
     def __urlChanged(self, url):
@@ -109,7 +204,8 @@ class WebBrowserPage(QWebEnginePage):
         """
         if not url.isEmpty() and url.scheme() == "eric" and \
                 not self.isJavaScriptEnabled():
-            self.setJavaScriptEnabled(True)
+            self.settings().setAttribute(QWebEngineSettings.JavascriptEnabled,
+                                         True)
             self.triggerAction(QWebEnginePage.Reload)
     
     @classmethod
@@ -215,19 +311,6 @@ class WebBrowserPage(QWebEnginePage):
             else:
                 QWebEnginePage.runJavaScript(self, script, callback)
     
-    def setJavaScriptEnabled(self, enable):
-        """
-        Public method to enable JavaScript.
-        
-        @param enable flag indicating the enabled state to be set
-        @type bool
-        """
-        if not self.url().isEmpty() and self.url().scheme() == "eric":
-            enable = True
-        
-        self.settings().setAttribute(QWebEngineSettings.JavascriptEnabled,
-                                     enable)
-    
     def isJavaScriptEnabled(self):
         """
         Public method to test, if JavaScript is enabled.
@@ -288,18 +371,30 @@ class WebBrowserPage(QWebEnginePage):
         """
         return WebHitTestResult(self, pos)
     
-    def __setupWebChannel(self):
+    def __setupWebChannelForUrl(self, url):
         """
         Private method to setup a web channel to our external object.
-        """
-        oldChannel = self.webChannel()
-        newChannel = QWebChannel(self)
-        newChannel.registerObject("eric_object", ExternalJsObject(self))
-        self.setWebChannel(newChannel)
         
-        if oldChannel:
-            del oldChannel.registeredObjects["eric_object"]
-            del oldChannel
+        @param url URL for which to setup the web channel
+        @type QUrl
+        """
+        channel = self.webChannel()
+        if channel is None:
+            channel = QWebChannel(self)
+            ExternalJsObject.setupWebChannel(channel, self)
+        
+        worldId = -1
+        if url.scheme() in ("eric", "qthelp"):
+            worldId = self.UnsafeJsWorld
+        else:
+            worldId = self.SafeJsWorld
+        if worldId != self.__channelWorldId:
+            self.__channelWorldId = worldId
+            try:
+                self.setWebChannel(channel, self.__channelWorldId)
+            except TypeError:
+                # pre Qt 5.7.0
+                self.setWebChannel(channel)
     
     def certificateError(self, error):
         """
@@ -374,6 +469,19 @@ class WebBrowserPage(QWebEnginePage):
         self.view().mainWindow().javascriptConsole().javaScriptConsoleMessage(
             level, message, lineNumber, sourceId)
     
+    ###########################################################################
+    ## Methods below implement safe browsing related functions
+    ###########################################################################
+    
+    def getSafeBrowsingStatus(self):
+        """
+        Public method to get the safe browsing status of the current page.
+        
+        @return flag indicating a safe site
+        @rtype bool
+        """
+        return not self.__badSite
+    
     ##################################################
     ## Methods below implement compatibility functions
     ##################################################
