@@ -372,7 +372,7 @@ def main():
     # start the event loop
     inMainLoop = True
     res = app.exec_()
-    logging.debug("Shutting down, result {0:d}".format(res))
+    logging.debug("Shutting down, result %d", res)
     logging.shutdown()
     sys.exit(res)
 
