@@ -342,7 +342,7 @@ def main(argv):
 
     global platBinDir
 
-    for opt, arg in optlist:
+    for opt, _arg in optlist:
         if opt == "-h":
             usage(0)
         if opt == "-y":
